package ziputils

import java.nio.ByteBuffer
import java.nio.ByteOrder

/**
 * Partial End of Central Directory record class.
 * Only supports data required by the backup tool.
 */
internal class EndOfCentralDirectoryRecord(
    val centralDirectoryOffset: UInt
) {
    fun eocd64Required(): Boolean =
        centralDirectoryOffset == 0xffffffffU

    companion object {
        const val SIGNATURE = 0x06054b50U
        const val SIZE = 22
        /**
         * Create EndOfCentralDirectoryRecord from raw byte data.
         * @throws InvalidDataException provided ByteArray is not a supported EOCD64.
         */
        @Throws(InvalidDataException::class)
        fun fromByteArray(data: ByteArray, offset: Int): EndOfCentralDirectoryRecord {
            if (data.size - offset < SIZE) {
                throw InvalidDataException("EOCD must be at least 22 bytes")
            }
            val buf = ByteBuffer.wrap(data, offset, SIZE).order(ByteOrder.LITTLE_ENDIAN)
            if (buf.getInt().toUInt() != SIGNATURE) {
                throw InvalidSignatureException("Invalid signature")
            }
            buf.position(offset + 16)
            return EndOfCentralDirectoryRecord(
                centralDirectoryOffset = buf.getInt().toUInt()
            )
        }
    }
}